import fsExtra from 'fs-extra';
import path from 'path';
import ts from 'typescript';
import { kJSOrTSFilepathRegex, kJSOrTSTestFilepathRegex, kJSQuoteGlobalRegex, kJSRelativeImportRegex, } from './constants.js';
export function isJSOrTSFilepath(filepath) {
    return kJSOrTSFilepathRegex.test(filepath);
}
export function isJSOrTSTestFilepath(filepath) {
    return kJSOrTSTestFilepathRegex.test(filepath);
}
export async function traverseAndProcessFilesInFolderpath(folderpath, handleFile) {
    const dirList = await fsExtra.readdir(folderpath, { withFileTypes: true });
    await Promise.all(dirList.map(async (entry) => {
        const entryPath = path.normalize(path.join(entry.path, entry.name));
        try {
            if (entry.isDirectory()) {
                await traverseAndProcessFilesInFolderpath(entryPath, handleFile);
            }
            else if (entry.isFile() && isJSOrTSFilepath(entryPath)) {
                const modifiedFile = await handleFile(entryPath);
                if (modifiedFile) {
                    console.log(`modified ${entryPath}`);
                }
            }
        }
        catch (error) {
            console.log(`error ${entryPath}`);
            console.error(error);
        }
    }));
}
export async function isFileOrFolder(filepath) {
    try {
        const info = await fsExtra.stat(filepath);
        return info.isFile() ? 'file' : info.isDirectory() ? 'folder' : undefined;
    }
    catch (error) {
        return undefined;
    }
}
export function isRelativeImportText(text) {
    return kJSRelativeImportRegex.test(text);
}
export function getImportText(node, sourceFile) {
    return node.getText(sourceFile).replaceAll(kJSQuoteGlobalRegex, '');
}
export function replaceNodeText(text, sourceFile, node, replacementText, offset) {
    const nodeText = node.getText(sourceFile);
    const trimmedNodeText = nodeText.trimStart();
    const start = node.getStart(sourceFile, /** includeJsDocComment */ false) +
        (nodeText.length - trimmedNodeText.length) +
        offset;
    const end = trimmedNodeText.length + start;
    const modifiedText = text.slice(0, start) + replacementText + text.slice(end);
    const newOffset = offset + (replacementText.length - trimmedNodeText.length);
    return { modifiedText, newOffset };
}
export function isRelativeImportOrExportNodeWithSpecifier(sourceFile, node) {
    return !!((ts.isExportDeclaration(node) || ts.isImportDeclaration(node)) &&
        node.moduleSpecifier &&
        ts.isStringLiteral(node.moduleSpecifier) &&
        isRelativeImportText(getImportText(node.moduleSpecifier, sourceFile)));
}
export function countCharacters(text, from, to, exp) {
    let totalCount = 0;
    let numCharsToFirstOccurrence = 0;
    for (let i = from; i < text.length && i < to; i++) {
        if (text[i] === exp) {
            totalCount += 1;
        }
        else {
            if (totalCount === 0) {
                numCharsToFirstOccurrence += 1;
            }
        }
    }
    return { totalCount, numCharsToFirstOccurrence };
}
//# sourceMappingURL=utils.js.map